library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity voltage_seq_control is


generic (USE_fixed_Time : integer range 0 to 1 := 0);


    port(SYS_CLK, SYS_RESET : in std_logic; 
      
    AVn12p0v_ENABLE : out std_logic; 
    AV12p0v_ENABLE  : out std_logic; 
    AV5p0v_ENABLE   : out std_logic;
    AV3p3v_ENABLE   : out std_logic; 
     

	all_supply_on, all_load_on : out std_logic;
	PwrUP: in std_logic;        --switch to power up the load supply
    Pdown: in std_logic;        --switch to shut down the load supply
    
    AVn12p0v_UNDERn10p8V, 		-- -12.0v supply is greater than -4.5v
    ACn12p0v_over1p5a,  		-- -12.0v supply current is greater than -1.5A
    AVn12p0v_load_over_n0p8v, 	-- -12.0v load is greater than -3.7v
    AVn12p0v_load_under_n10p8v,	-- -12.0v load is less than -0.8v 

    AV12p0v_OVER11p4V,           -- 12.0v supply is greater than 4.5v
    AC12p0v_over2p0a,   -- 12.0v supply current is greater than 2A
    AV12p0v_load_over_11p4v,      -- 12.0 load is greater than 3.7v
    AV12p0v_load_under_0p8v,     -- 12.0 load is less than 0.8v
    
    AV5p0v_OVER4p75V,          -- 5.0v supply is greater than 1.425v
    AV5p0v_load_over_4p75v,     -- 5.0v load is greater than 0.975v
    AV5p0v_load_under_0p8v,       -- 5.0v load is less than 0.5v    

    AV3p3v_OVER3p17V,            -- 3.3v supply is greater than 3.6v
    AV3p3v_load_over_3p17v,       -- 3.3v load is greater than 2.8v
    AV3p3v_load_under_0p8v      -- 3.3v load is less than 0.8v 

    

      
    : in std_logic);

end voltage_seq_control;

architecture DEF_ARCH of voltage_seq_control is 

component count12
  port(Clock : in std_logic;
       Q : out std_logic_vector(11 downto 0);
       Aclr : in std_logic;
       Tcnt : out std_logic);
end component;


type state_type is (warmup, 
					state1, --Check -12v supply flag 
					state2, --Check 12v supply flag 
					state3, --Check 5.0v supply flag 
 				    state4, --Check 3.3v supply flag 
					supply_on,
                    avn12_on, av12_on, av50_on, av33_on,
                    monitoring,
                    avn12_off, av12_off, av50_off, av33_off);

signal state : state_type;
attribute syn_state_machine : boolean;
attribute syn_state_machine of state : signal is true;  

signal timeup : std_logic; --terminal count for the 12-bit counter
signal aclr_cntr : std_logic; --terminal count for the 12-bit counter

signal avn12_supply_on, av12_supply_on, av50_supply_on, av33_supply_on: std_logic; 

begin


	my_count : count12
      port map(Aclr => aclr_cntr, Clock => SYS_CLK, Q => OPEN, Tcnt => timeup );



	-- main state machine
	state_set: process (SYS_RESET, SYS_CLK, Pdown, AVn12p0v_UNDERn10p8V, AV12p0v_OVER11p4V, AV5p0v_OVER4p75V, AV3p3v_OVER3p17V,
	AVn12p0v_load_over_n0p8v, AVn12p0v_load_under_n10p8v,   AV12p0v_load_over_11p4v,AV12p0v_load_under_0p8v,AV3p3v_load_over_3p17v,
	AV3p3v_load_under_0p8v,AV5p0v_load_over_4p75v,  AV5p0v_load_under_0p8v)

	begin
		if SYS_RESET = '0' then
           state <= warmup;           
           AVn12p0v_ENABLE <= '0';  
           AV12p0v_ENABLE <= '0';  
           AV3p3v_ENABLE <= '0';  
           AV5p0v_ENABLE <= '0';
 		   avn12_supply_on <= '0';
		   av12_supply_on <= '0';
		   av50_supply_on <= '0';
           av33_supply_on <= '0'; 
		   aclr_cntr <= '0';
 	
		elsif (SYS_CLK'event and SYS_CLK = '1') then
            
            ---------------------------------------------------------------
            --Sequencing Controlled by Fixed Delay Between Power Supplies--
            ---------------------------------------------------------------
            if (USE_fixed_Time = 1) then	            
                case state is
        			when warmup =>                      --Fusion Device Is Powered Up
        					AVn12p0v_ENABLE <= '0';  
                            AV12p0v_ENABLE <= '0';  
                            AV5p0v_ENABLE <= '0';
                            AV3p3v_ENABLE <= '0';  
                            all_supply_on <= '0';
        					all_load_on <= '0';
					
       						state <= state1;
            
                     when state1 =>                     --Monitor -12VDC Power Supply
        			        if AVn12p0v_UNDERn10p8V = '1' then
        						avn12_supply_on <= '1'; 
                            end if;
							state <= state2;

					when state2 =>                      --Monitor +12VDC Power Supply
							if AV12p0v_OVER11p4V = '1' then
        						av12_supply_on <= '1'; 
                            end if;
							state <= state3;
            
                    when state3 =>                      --Monitor +5VDC Power Supply
							if AV5p0v_OVER4p75V = '1' then
        						av50_supply_on <= '1'; 
                            end if;
                                        
                    when state4 =>                      --Monitor +3.3VDC Power Supply
        			        if AV3p3v_OVER3p17V = '1' then
        						av33_supply_on <= '1'; 
                            end if;
							state <= state4;
            
                    

							if (avn12_supply_on = '1' and av12_supply_on = '1' and av50_supply_on = '1' and av33_supply_on = '1') then
                                state <= supply_on;
                                all_supply_on <= '1';
        					else
        						state <= state1;
                            end if;


                    when supply_on =>                   --Start Power Up Sequence
                            if PwrUp = '1' then
        						state <= avn12_on;
        					else
        						state <= supply_on;
                            end if;
	
                    when avn12_on =>                    --Turn On -12VDC After Fixed Delay
                            if timeup = '1' then
        						state <= av12_on;
        					else
        						state <= avn12_on;
                            end if;
                            AVn12p0v_ENABLE <= '1';
                            aclr_cntr <= '1'; 
            
                    when av12_on =>                     --Turn On +12VDC After Fixed Delay
        			        if timeup = '1' then
        						state <= av50_on;
        					else
        						state <= av12_on;
                            end if;
                            AV12p0v_ENABLE <= '1';
                            aclr_cntr <= '1';  
            
                    when av50_on =>                     --Turn On +5VDC After Fixed Delay
        			        if timeup = '1' then
        						state <= av33_on;
        					else
        						state <= av50_on;
                            end if;
                            AV5p0v_ENABLE <= '1';
                            aclr_cntr <= '1';  

            
                    when av33_on =>                     --Turn On +3.3VDC After Fixed Delay
        			        if timeup = '1' then
        						state <= monitoring;
        					else
        						state <= av33_on;
                            end if;
                             AV3p3v_ENABLE <= '1';
                             aclr_cntr <= '1';  
            
                    when monitoring =>                  --Switch to Power Down Sequence
        			        if Pdown = '1' then
        						state <= av33_off;
        					else
        						state <= monitoring;
                            end if;
        					all_load_on <= '1';
                            aclr_cntr <=  '0';

                    when av33_off =>                    --Turn Off +3.3VDC After Fixed Delay
        			        if timeup = '1' then
        						state <= av50_off;
        					else
        						state <= av33_off;
                            end if;
                            AV3p3v_ENABLE <= '0';

                            aclr_cntr <=  '1';
            
                    when av50_off =>                    --Turn Off +5VDC After Fixed Delay
        			        if timeup = '1' then
        						state <= av12_off;
        					else
        						state <= av50_off;
                            end if;
                            AV5p0v_ENABLE <= '0';

                            aclr_cntr <=  '1'; 
            
                    when av12_off =>                    --Turn Off +12VDC After Fixed Delay
        			        if timeup = '1' then
        						state <= avn12_off;
        					else
        						state <= av12_off;
                            end if;
                            AV12p0v_ENABLE <= '0';

                            aclr_cntr <=  '1'; 
            
                    when avn12_off =>                   --Turn Off -12VDC After Fixed Delay
        			    --    if timeup = '1' then
        				--		state <= warmup;
        				--	else
        				--		state <= avn50_off;
                        --    end if;
                            AVn12p0v_ENABLE <= '0';

                            aclr_cntr <=  '0';
                 end case;
         
               ---------------------------------------------------------------
               -- Sequencing Controlled by Threshold Level of Power Supplies--
               ---------------------------------------------------------------
               else
              

                case state is
        			when warmup =>                              --Fusion Device Is Powered Up
        					AVn12p0v_ENABLE <= '0';  
                            AV12p0v_ENABLE <= '0';  
                            AV5p0v_ENABLE <= '0';
                            AV3p3v_ENABLE <= '0';  
                            all_supply_on <= '0';
        					all_load_on <= '0';
				
       						state <= state1;
            
                     when state1 =>                             --Monitor -12VDC Power Supply
        			        if AVn12p0v_UNDERn10p8V = '1' then
        						avn12_supply_on <= '1'; 
                            end if;
							state <= state2;

					when state2 =>                              --Monitor +12VDC Power Supply
							if AV12p0v_OVER11p4V = '1' then
        						av12_supply_on <= '1'; 
                            end if;
							state <= state3;
            
                    when state3 =>                              --Monitor +5VDC Power Supply
        			        if AV5p0v_OVER4p75V = '1' then
        						av50_supply_on <= '1'; 
                            end if;
                            state <= state4;
            
                    when state4 =>                              --Monitor +3.3VDC Power Supply
							if AV3p3v_OVER3p17V = '1' then
        						av33_supply_on <= '1'; 
                            end if;

							if (avn12_supply_on = '1' and av12_supply_on = '1' and av50_supply_on = '1' and av33_supply_on = '1') then
 								state <= supply_on;
                                all_supply_on <= '1';
        					else
        						state <= state1;
                            end if;

		            when supply_on =>                           --Start Power Up Sequence
        			        if PwrUp = '1' then
        						state <= avn12_on;
        					else
        						state <= supply_on;
                            end if;        
 
                    when avn12_on =>                            --Turn On -12VDC After Fixed Delay
        			        if AVn12p0v_load_under_n10p8v = '1' then
        						state <= av12_on;
        					else
        						state <= avn12_on;
                            end if;
                            AVn12p0v_ENABLE <= '1'; 
         
                    when av12_on =>                             --Turn On +12VDC After Fixed Delay
        			        if AV12p0v_load_over_11p4v = '1' then
        						state <= av50_on;
        					else
        						state <= av12_on;
                            end if;
                            AV12p0v_ENABLE <= '1';            

                    when av50_on =>                             --Turn On +5VDC After Fixed Delay
        			        if AV5p0v_load_over_4p75v = '1' then
        						state <= av33_on;
        					else
        						state <= av50_on;
                            end if;
                             AV5p0v_ENABLE <= '1';                     
                                        
                    when av33_on =>                             --Turn On +3.3VDC After Fixed Delay
        			        if AV3p3v_load_over_3p17v = '1' then
        						state <= monitoring;
        					else
        						state <= av33_on;
                            end if;
                            AV3p3v_ENABLE <= '1';  
           

                    
          

                    when monitoring =>                          --Switch to Power Down Sequence
        			        if Pdown = '1' then
        						state <= av33_off;
        					else
        						state <= monitoring;
                            end if;
        					all_load_on <= '1';


                    when av33_off =>                            --Turn Off +3.3VDC After Fixed Delay
        			        if AV3p3v_load_over_3p17v = '0' then
        						state <= av50_off;
        					else
        						state <= av33_off;
                            end if;
                            AV3p3v_ENABLE <= '0';
          

                    when av50_off =>                            --Turn Off +5VDC After Fixed Delay
        			        if AV5p0v_load_over_4p75v = '0' then
        						state <= av12_off;
        					else
        						state <= av50_off;
                            end if;
                            AV5p0v_ENABLE <= '0'; 
            

                    when av12_off =>                            --Turn Off +12VDC After Fixed Delay
        			        if AV12p0v_load_over_11p4v = '0' then
        						state <= avn12_off;
        					else
        						state <= av12_off;
                            end if;
                            AV12p0v_ENABLE <= '0'; 

            
                    when avn12_off =>                           --Turn Off -12VDC After Fixed Delay
        			        if AVn12p0v_load_under_n10p8v = '0' then
        						state <= warmup;
        					else
        						state <= avn12_off;
                            end if;
                            AVn12p0v_ENABLE <= '0';

                 end case;

            end if; 


        end if;


end process;

end DEF_ARCH; 
