library ieee, fusion;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.math_real.all;
use fusion.analog_io.all;



-- ********************************************************************************************
ENTITY testbench IS
generic (USE_fixed_Time : integer range 0 to 1 := 0);
END testbench;

architecture stimuli of testbench is

constant WIDTH : integer := 32;


	-- System Clock cycle (in ns)
constant	SYS_CLK_CYCLE		:	integer	:= 20
;
---------------------------------------------------------------------
-- Run simulation for given number of clock cycles
-- (procedure description here in architecture to use SYS_CLK_CYCLE generic)
---------------------------------------------------------------------
procedure cyc	(
	constant	c:			in	integer range 0 to 65536) is
begin
	cloop: for i in 1 to c loop
		wait for (SYS_CLK_CYCLE * 1 ns);
	end loop cloop;
end cyc;
---------------------------------------------------------------------
-- Wait for signals
---------------------------------------------------------------------
procedure sigwait	(
	signal		sig:		in		std_logic;
	constant	val:		in		std_logic
) is
begin
	while (sig /= val) loop
		cyc(1);
	end loop;
	cyc(1);
end sigwait;

procedure sigwait	(
	signal		sig:	in		std_logic_vector;
	constant	val:	in		std_logic_vector
) is
	variable	valstd:			std_logic_vector(sig'range);
begin
	for i in sig'range loop
		if (val(i) = '1') then
			valstd(i) := '1';
		else
			valstd(i) := '0';
		end if;
	end loop;

	while (sig /= valstd) loop
		cyc(1);
	end loop;
	cyc(1);
end sigwait;


component my_Top

generic (USE_fixed_Time : integer range 0 to 1 := 1); 

port(SYS_CLK_20MHZ : in std_logic; --input main clock of 20Mhz
    SYS_RESET : in std_logic; --active low reset
     
    DATAVALID : out std_logic; 
    INIT_POWER_UP : in std_logic;
 	INIT_DONE_OUTPUT : out std_logic; 
    
    ASSC_CHLATD, ASSC_CHSAT, ASSC_WAIT, ASSC_DONE : out std_logic; 
    
    VAREF : inout std_logic := 'Z'; 
    
    PwrUP, --switch to power up the load supply
    PDOWN  --switch to shut down the load supply
    : in std_logic;
    
    AVN12P0V,    --     -12v supply to analog voltage input
    AV12P0V,     --     12v supply to analog voltage input
    AV5P0V,     --      5.0v supply to analog voltage input
    AV3P3V,     --      3.3v supply to analog voltage input
    ACN12P0V,    --     Current on -12v supply to analog current input
    AC12P0V,     --     Current on 12v supply to analog current input
    AVN12P0V_LOAD, --   -12v load supply to analog voltage input
    AV12P0V_LOAD,  --   12v load supply to analog voltage input
    AV5P0V_LOAD,  --    5.0v load supply to analog voltage input
    AV3P3V_LOAD   --    3.3v load supply to analog voltage input
    : in std_logic; 
    
    ALL_SUPPLY_ON : out std_logic; --Flag used to indiacte all supply are on 
    ALL_LOAD_ON : out std_logic;   --Flag used to indiacte all load supply are on 

    AVN12P0V_ON, --input -12v supply to analog quad
    AV12P0V_ON,  --input 12v supply to analog quad
    AV5P0V_ON,  --input 5.0v supply to analog quad
    AV3P3V_ON : out std_logic); --input 3.3v supply to analog quad
    
end component;
    


constant CLOCK_PERIOD : time := 20 ns;


signal DUTY_CYCLE : time := 25 ns;
signal STATE_VARIABLE : real := 0.0;


signal asscdone: std_logic;
signal datavalid: std_logic;
signal init_done: std_logic;
signal varef:std_logic;

signal sysclk: std_logic;

signal reset : std_logic;
signal init_power_up: std_logic;

signal p_varef_i 		: real;
signal p_vagnd_i 		: real;
signal temp_refn_i		: real;
signal temp_refp_i		: real;
signal die_temp_i 		: real;

signal p_ain_i_c 			: real;
signal p_ain_i_cn 			: real;
--signal p_ain_i_vl 			: real;

signal p_ain_i_v33 			: real;
signal p_ain_i_v50 			: real;
signal p_ain_i_v12 			: real;
signal p_ain_i_vn12			: real;

signal p_ain_i_v33_load 			: real;
signal p_ain_i_v50_load 			: real;
signal p_ain_i_v12_load 			: real;
signal p_ain_i_vn12_load			: real;

-- lachting switches

type twoDimensions is array (4 downto 1) of std_logic_vector(WIDTH-1 downto 0);

signal p_ain_0_v33 			: std_logic;
signal p_ain_0_v50 			: std_logic;
signal p_ain_0_v12 			: std_logic;
signal p_ain_0_vn12			: std_logic;
signal p_ain_o_c            : std_logic;
signal p_ain_o_cn            : std_logic;

signal p_ain_0_v33_load 			: std_logic;
signal p_ain_0_v50_load 			: std_logic;
signal p_ain_0_v12_load 			: std_logic;
signal p_ain_0_vn12_load			: std_logic;

signal p : integer := 0;


signal AVN12P0V_LOAD : std_logic; 
signal AV12P0V_LOAD : std_logic; 
signal AV5P0V_LOAD : std_logic; 
signal AV3P3V_LOAD : std_logic;


signal AVN12P0V_ON : std_logic;
signal AV12P0V_ON : std_logic;
signal AV5P0V_ON : std_logic;
signal AV3P3V_ON : std_logic;

signal PDOWN : std_logic;
signal PwrUp : std_logic;

--signal assc_done: std_logic;
signal assc_wait: std_logic;
signal assc_chsat: std_logic;
signal assc_chlatd: std_logic;

signal ALL_SUPPLY_ON, ALL_LOAD_ON : std_logic;

-- ********************************************************************************************

begin

serial_p_ain_i_c : process
     begin
       wait on p_ain_i_c;
       if ( INIT_DONE = '1') then 
         drive_analog_input ( real(p_ain_i_c), p_ain_o_c );
       end if;
     end process serial_p_ain_i_c;
     

serial_p_ain_i_cn : process
     begin
       wait on p_ain_i_cn;
       if ( INIT_DONE = '1') then 
         drive_analog_input ( real(p_ain_i_cn), p_ain_o_cn );
       end if;
     end process serial_p_ain_i_cn;


serial_p_ain_i_v33 : process
     begin      
       wait on p_ain_i_v33;
       if ( INIT_DONE = '1') then
         drive_analog_input ( real(p_ain_i_v33), p_ain_0_v33 );
       end if;
end process serial_p_ain_i_v33;


serial_p_ain_i_v50 : process
     begin
       wait on p_ain_i_v50;
       if ( INIT_DONE = '1') then
         drive_analog_input ( real(p_ain_i_v50), p_ain_0_v50 );
       end if;
end process serial_p_ain_i_v50;


serial_p_ain_i_v12 : process
     begin       
       wait on p_ain_i_v12;
       if ( INIT_DONE = '1') then
         drive_analog_input ( real(p_ain_i_v12), p_ain_0_v12 );
       end if;
end process serial_p_ain_i_v12;


serial_p_ain_i_vn12 : process
     begin     
       wait on p_ain_i_vn12;
       if ( INIT_DONE = '1') then
         drive_analog_input ( real(p_ain_i_vn12), p_ain_0_vn12 );
       end if;
end process serial_p_ain_i_vn12;


serial_p_ain_i_v33_load : process
     begin      
       wait on p_ain_i_v33_load;
       if ( INIT_DONE = '1') then
         drive_analog_input ( real(p_ain_i_v33_load), p_ain_0_v33_load );
       end if;
end process serial_p_ain_i_v33_load;


serial_p_ain_i_v50_load : process
     begin
       wait on p_ain_i_v50_load;
       if ( INIT_DONE = '1') then
         drive_analog_input ( real(p_ain_i_v50_load), p_ain_0_v50_load );
       end if;
end process serial_p_ain_i_v50_load;


serial_p_ain_i_v12_load : process
     begin       
       wait on p_ain_i_v12_load;
       if ( INIT_DONE = '1') then
         drive_analog_input ( real(p_ain_i_v12_load), p_ain_0_v12_load );
       end if;
end process serial_p_ain_i_v12_load;


serial_p_ain_i_vn12_load : process
     begin     
       wait on p_ain_i_vn12_load;
       if ( INIT_DONE = '1') then
         drive_analog_input ( real(p_ain_i_vn12_load), p_ain_0_vn12_load );
       end if;
end process serial_p_ain_i_vn12_load;

  
num : process
begin

  STATE_VARIABLE <= STATE_VARIABLE + 1.0;

  wait for CLOCK_PERIOD;

end process num;


-- SYSTEM RESET

rst : process
begin

  reset <= '0';
  wait for 300 ns;
  reset <= '1';
  wait until (STATE_VARIABLE < 1.0);

end process rst;

-- POWERUP AFTER RESET

pup : process
begin

  init_power_up <= '0';
  wait for 1 ns;
  init_power_up <= '1';
  wait until (STATE_VARIABLE < 1.0);

end process pup;


-- System Clock

clock : process
begin

  sysclk <= '0';
  wait for DUTY_CYCLE;
  sysclk <= '1';
  wait for DUTY_CYCLE;

end process clock;

TOP_inst: my_TOP 
generic map ( USE_fixed_Time => USE_fixed_Time)
port map (
INIT_DONE_OUTPUT=> INIT_DONE,
INIT_POWER_UP => init_power_up, 
SYS_RESET => reset, 
SYS_CLK_20MHZ => sysclk,

ALL_SUPPLY_ON => ALL_SUPPLY_ON,
ALL_LOAD_ON => ALL_LOAD_ON,

AVN12P0V_ON => AVN12P0V_ON,
AV12P0V_ON => AV12P0V_ON,
AV5P0V_ON => AV5P0V_ON,
AV3P3V_ON => AV3P3V_ON,

AVN12P0V => p_ain_0_vn12, 
ACN12P0V => p_ain_o_cn,
AV12P0V => p_ain_0_v12, 
AC12P0V => p_ain_o_c, 
AV5P0V => p_ain_0_v50, 
AV3P3V => p_ain_0_v33,

AVN12P0V_LOAD => p_ain_0_vn12_load, 
AV12P0V_LOAD => p_ain_0_v12_load, 
AV5P0V_LOAD => p_ain_0_v50_load, 
AV3P3V_LOAD => p_ain_0_v33_load, 

PDOWN => PDOWN,
PwrUP => PwrUP,

VAREF => varef,
DATAVALID => datavalid, 
ASSC_DONE => asscdone, 
ASSC_WAIT => ASSC_WAIT, 
ASSC_CHSAT => ASSC_CHSAT, 
ASSC_CHLATD => ASSC_CHLATD ); 

-- ********************************************************************************************

  stimulate : process

    variable i_v 		: integer;

  begin

 
  -- ********************************************************************************************
  -- ANALOG SECTION 
  -- ********************************************************************************************
  -- using the syntesis version of interface_pack this values will be mapped
  -- either to std_logic HI '1' or std_logic LO '0'
  -- ********************************************************************************************

    p_varef_i <= 2.56;
    p_vagnd_i <= 0.0;
    die_temp_i <= 0.7;
    temp_refn_i <= 0.0;
    temp_refp_i <= 3.3;

PDOWN <= '0';
PwrUP <= '0';
    
    --- initial values
    p_ain_i_c  <= 1.99;
    p_ain_i_cn  <= -1.99;
    p_ain_i_v33  <= 2.00;
    p_ain_i_v50  <= 3.00;
    p_ain_i_v12  <= 10.00;
    p_ain_i_vn12  <= -10.00;

    p_ain_i_v33_load  <= 0.00;
    p_ain_i_v50_load  <= 0.00;
    p_ain_i_v12_load  <= 0.00;
    p_ain_i_vn12_load  <= 0.00;
    
    
    -- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

    --- Measure input voltage for 20 cycles
    p_ain_i_v33  <= 1.00;
    p_ain_i_v50  <= 3.00;
    p_ain_i_v12  <= 8.00;
    p_ain_i_vn12  <= -8.00;
    p_ain_i_c  <= 7.99; --Dummy value for AC 
    p_ain_i_cn  <= -7.99; --Dummy value for AC
    
    -- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;
	
    --- Measure input voltage for 20 cycles
    p_ain_i_v33  <= 2.00;
    p_ain_i_v50  <= 3.00;
    p_ain_i_v12  <= 10.00;
    p_ain_i_vn12  <= -10.00;
    P_ain_i_c  <= 9.29; 
    P_ain_i_cn  <= -9.29;
    
    -- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

    --- Measure input voltage for 20 cycles
    p_ain_i_v33  <= 2.50;
    p_ain_i_v50  <= 5.00;
    p_ain_i_v12  <= 12.00;
    p_ain_i_vn12  <= -12.00;
    p_ain_i_c  <= 11.45;
    p_ain_i_cn  <= -11.45;

        -- wait for ASSCDONE signal to subside 20 times 
    	for i in 1 to 20 loop
    		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

    --- Measure input voltage for 20 cycles
    p_ain_i_v33  <= 1.00;
    p_ain_i_v50  <= 5.00;
    p_ain_i_v12  <= 12.00;
    p_ain_i_vn12  <= -12.00;
    p_ain_i_c  <= 11.90; 
    p_ain_i_cn  <= -11.90;


    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

    --- Measure input voltage for 20 cycles
    p_ain_i_v33  <= 3.30;
    p_ain_i_v50  <= 5.00;
    p_ain_i_v12  <= 12.00;
    p_ain_i_vn12  <= -12.00;
    p_ain_i_c  <= 11.99; --Dummy value for AC12V
    p_ain_i_cn  <= -11.99; --Dummy value for ACN12V
    
    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

wait until ALL_SUPPLY_ON ='1' ; --all input voltage are on
wait for 1ms;
PwrUP <= '1';

if (USE_fixed_Time = 1) then
  
        wait until ALL_LOAD_ON ='1' ; --all load side are on
        wait for 4ms;

        PDOWN <= '1';
        	for i in 1 to 20 loop
        		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
        	end loop;

 
            wait for 5ms;
            PwrUP <= '0';
else

    --- Measure load voltage for 20 cycles
    p_ain_i_v33_load  <= 0.00;
    p_ain_i_v50_load  <= 0.00;
    p_ain_i_v12_load  <= 0.00;
    p_ain_i_vn12_load  <= -12.00;

    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

    --- Measure load voltage for 20 cycles
    p_ain_i_v33_load  <= 0.00;
    p_ain_i_v50_load  <= 0.00;
    p_ain_i_v12_load  <= 12.00;
    p_ain_i_vn12_load  <= -12.00;

    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

    --- Measure load voltage for 20 cycles
    p_ain_i_v33_load  <= 0.00;
    p_ain_i_v50_load  <= 5.0;
    p_ain_i_v12_load  <= 12.00;
    p_ain_i_vn12_load  <= -12.00;

    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

    --- Measure load voltage for 20 cycles
    p_ain_i_v33_load  <= 3.3;
    p_ain_i_v50_load  <= 5.0;
    p_ain_i_v12_load  <= 12.00;
    p_ain_i_vn12_load  <= -12.00;

    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;


wait until ALL_LOAD_ON ='1' ; --all load side are on
wait for 4ms;

PDOWN <= '1';
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

    --- Measure load voltage for 20 cycles
    p_ain_i_v33_load  <= 0.00;
    p_ain_i_v50_load  <= 5.00;
    p_ain_i_v12_load  <= 12.00;
    p_ain_i_vn12_load  <= -12.00;

    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop;

    --- Measure load voltage for 20 cycles
    p_ain_i_v33_load  <= 0.00;
    p_ain_i_v50_load  <= 0.00;
    p_ain_i_v12_load  <= 12.00;
    p_ain_i_vn12_load  <= -12.00;

    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop; 

    --- Measure load voltage for 20 cycles
    p_ain_i_v33_load  <= 0.00;
    p_ain_i_v50_load  <= 0.00;
    p_ain_i_v12_load  <= 12.00;
    p_ain_i_vn12_load  <= -12.00;

    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop; 

    --- Measure load voltage for 20 cycles
    p_ain_i_v33_load  <= 0.00;
    p_ain_i_v50_load  <= 0.00;
    p_ain_i_v12_load  <= 0.00;
    p_ain_i_vn12_load  <= -0.00;

    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop; 


    --- Measure load voltage for 20 cycles
    p_ain_i_v33_load  <= 0.00;
    p_ain_i_v50_load  <= 0.00;
    p_ain_i_v12_load  <= 0.00;
    p_ain_i_vn12_load  <= -0.00;

    	-- wait for ASSCDONE signal to subside 20 times 
	for i in 1 to 20 loop
		sigwait(ASSCDONE,'1'); sigwait(ASSCDONE,'0');
	end loop; 

    wait for 5ms;
    PwrUP <= '0';

end if;

  end process stimulate;

-- ********************************************************************************************

end stimuli;

-- ********************************************************************************************
